@@ -1,13 +1,12 @@
 #
 # This file is part of Config-Model
 #
-# This software is Copyright (c) 2010 by Dominique Dumont, Krzysztof Tyszecki.
+# This software is Copyright (c) 2011 by Dominique Dumont, Krzysztof Tyszecki.
 #
 # This is free software, licensed under:
 #
 #   The GNU Lesser General Public License, Version 2.1, February 1999
 #
-
 #    Copyright (c) 2010 Dominique Dumont.
 #
 #    This file is part of Config-Model.
@@ -28,14 +27,14 @@
 
 package Config::Model::Backend::Any ;
 BEGIN {
-  $Config::Model::Backend::Any::VERSION = '1.226';
+  $Config::Model::Backend::Any::VERSION = '1.250';
 }
 
 use Carp;
 use strict;
 use warnings ;
 use Config::Model::Exception ;
-use Moose ;
+use Any::Moose ;
 
 use File::Path;
 use Log::Log4perl qw(get_logger :levels);
@@ -75,6 +74,16 @@ sub read_global_comments {
 
     while (defined ( $_ = shift @$lines ) ) {
         next if /^$cc$cc/ ; # remove comments added by Config::Model
+        unshift @$lines,$_;
+        last;
+    }
+    while (defined ( $_ = shift @$lines ) ) {
+        next if /^\s*$/ ; # remove empty lines
+        unshift @$lines,$_;
+        last;
+    }
+
+    while (defined ( $_ = shift @$lines ) ) {
         chomp ;
 
         my ($data,$comment) = split /\s*$cc\s?/ ;
@@ -93,7 +102,72 @@ sub read_global_comments {
     }
 }
 
-no Moose ;
+sub associates_comments_with_data {
+    my $self = shift ;
+    my $lines = shift ;
+    my $cc = shift ; # comment character
+
+    my @result ;
+    my @comments ;
+    foreach (@$lines) {
+        next if /^$cc$cc/ ;		  # remove comments added by Config::Model
+        chomp ;
+
+        my ($data,$comment) = split /\s*$cc\s?/ ;
+        push @comments, $comment        if defined $comment ;
+
+        next unless defined $data ;
+        $data =~ s/^\s+//g;
+        $data =~ s/\s+$//g;
+
+        if ($data) {
+            my $note = '';
+            $note = join("\n",@comments) if @comments ;
+            $logger->debug("associates_comments_with_data: '$note' with '$data'");
+            push @result, [  $data , $note ] ;
+            @comments = () ;
+        }
+    }
+    
+    return wantarray ? @result : \@result ;
+   
+}
+
+sub write_global_comment {
+    my ($self,$ioh,$cc) = @_ ;
+
+    my $res = "$cc$cc This file was written by Config::Model\n"
+            . "$cc$cc You may modify the content of this file. Configuration \n"
+            . "$cc$cc modifications will be preserved. Modifications in\n"
+            . "$cc$cc comments may be mangled.\n\n" ;
+
+    # write global comment
+    my $global_note = $self->node->annotation ;
+    if ($global_note) {
+        map { $res .= "$cc $_\n" } split /\n/,$global_note ;
+        $res .= "\n" ;
+    }
+
+    $ioh->print ($res) if defined $ioh ;
+    return $res ;
+}
+
+sub write_data_and_comments {
+    my ($self,$ioh,$cc, @data_and_comments) = @_ ;
+
+    my $res  = '' ;
+    while (@data_and_comments) {
+        my ($d,$c) = splice @data_and_comments,0,2;
+        if ($c) {
+            map { $res .= "$cc $_\n" } split /\n/,$c ;
+        }
+        $res .= "$d\n" if defined $d;
+    }
+    $ioh->print ($res) if defined $ioh ;
+    return $res ;
+}
+
+no Any::Moose ;
 __PACKAGE__->meta->make_immutable ;
 
 1;
@@ -106,12 +180,12 @@ Config::Model::Backend::Any - Virtual class for other backends
 
 =head1 VERSION
 
-version 1.226
+version 1.250
 
 =head1 SYNOPSIS
 
  package Config::Model::Backend::Foo ;
- use Moose ;
+ use Any::Moose ;
  use Log::Log4perl qw(get_logger :levels);
 
  extends 'Config::Model::Backend::Any';
@@ -174,7 +248,7 @@ version 1.226
     return 1;
  }
 
- no Moose ;
+ no Any::Moose ;
  __PACKAGE__->meta->make_immutable ;
 
 =head1 DESCRIPTION
@@ -196,7 +270,7 @@ L<Config::Model::Node>.
 
 =head2 annotation
 
-Whether the backend supports to read and write annotation. Default i s
+Whether the backend supports to read and write annotation. Default is
 0. Override if your backend supports annotations
 
 =head1 Methods
@@ -204,7 +278,37 @@ Whether the backend supports to read and write annotation. Default i s
 =head2 read_global_comments( lines , comment_char)
 
 Read the global comments (i.e. the first block of comments until the first blank or non comment line) and
-store them as root node annotation. lines is an array ref containing file lines.
+store them as root node annotation. The first parameter (C<lines>)
+ is an array ref containing file lines.
+
+=head2 associates_comments_with_data ( lines , comment_char)
+
+This method will extract comments from the passed lines and associate 
+them with actual data found in the file lines. Data is associated with 
+comments preceding or on the same line as the data. Returns a list of
+[ data, comment ] .
+
+Example:
+
+  # Foo comments
+  foo= 1
+  Baz = 0 # Baz comments
+
+will return 
+
+  ( [  'foo= 1', 'Foo comments'  ] , [ 'Baz = 0' , 'Baz comments' ] )
+
+=head2 write_global_comments( io_handle , comment_char)
+
+Write global comments from configuration root annotation into the io_handle (if defined).
+Returns the string written to the io_handle.
+
+=head2 write_data_and_comments( io_handle , comment_char , data1, comment1, data2, comment2 ...)
+
+Write data and comments in the C<io_handle> (if defined). Comments are written before the data.
+Returns the string written to the io_handle. If a data is undef, the comment will be written on its own
+line.
+
 =head1 AUTHOR
 
 Dominique Dumont, (ddumont at cpan dot org)
@@ -216,4 +320,4 @@ L<Config::Model::AutoRead>,
 L<Config::Model::Node>, 
 L<Config::Model::Backend::Yaml>, 
 
-=cut
\ No newline at end of file
+=cut
