@@ -1,7 +1,7 @@
 #
 # This file is part of Config-Model
 #
-# This software is Copyright (c) 2010 by Dominique Dumont, Krzysztof Tyszecki.
+# This software is Copyright (c) 2011 by Dominique Dumont, Krzysztof Tyszecki.
 #
 # This is free software, licensed under:
 #
@@ -9,9 +9,9 @@
 #
 package Config::Model::Backend::Fstab ;
 BEGIN {
-  $Config::Model::Backend::Fstab::VERSION = '1.226';
+  $Config::Model::Backend::Fstab::VERSION = '1.250';
 }
-use Moose ;
+use Any::Moose ;
 use Carp ;
 use Log::Log4perl qw(get_logger :levels);
  
@@ -23,13 +23,12 @@ sub suffix { return '' ; }
 
 sub annotation { return 1 ;}
 
-my %opt_r_translate 
-  = (
-     ro => 'rw=0',
-     rw => 'rw=1',
-     bsddf => 'statfs_behavior=bsddf',
-     minixdf => 'statfs_behavior=minixdf',
-    ) ;
+my %opt_r_translate = (
+    ro => 'rw=0',
+    rw => 'rw=1',
+    bsddf => 'statfs_behavior=bsddf',
+    minixdf => 'statfs_behavior=minixdf',
+) ;
 
 sub read {
     my $self = shift ;
@@ -51,54 +50,45 @@ sub read {
     # try to get global comments (comments before a blank line)
     $self->read_global_comments(\@lines,'#') ;
 
-    my @comments ;
-    foreach (@lines) {
-        next if /^##/ ;		  # remove comments added by Config::Model
-        chomp ;
-        s/\s+$//;
+    my @assoc = $self->associates_comments_with_data( \@lines, '#' );
+    foreach my $item (@assoc) {
+        my ( $data, $comment ) = @$item;
+        $logger->trace("fstab read data '$data' comment '$comment'");
 
-        my ($data,$comment) = split /\s*#\s?/ ;
+        my ( $device, $mount_point, $type, $options, $dump, $pass ) =
+          split /\s+/, $data;
 
-        push @comments, $comment        if defined $comment ;
-        $logger->debug("Fstab: line $. '$_'\n");
+        my $swap_idx = 0;
+        my $label = $device =~ /LABEL=(\w+)$/ ? $1
+          : $type eq 'swap' ? "swap-" . $swap_idx++
+          :                   $mount_point;
 
-        if (defined $data and $data ) {
-            my ($device,$mount_point,$type,$options, $dump, $pass) = split /\s+/,$data ;
+        my $fs_obj = $self->node->fetch_element('fs')->fetch_with_id($label);
 
-            my $swap_idx = 0;
-            my $label = $device =~ /LABEL=(\w+)$/  ? $1 
-                      : $type eq 'swap'            ? "swap-".$swap_idx++ 
-                      :                              $mount_point; 
-
-            my $fs_obj = $self->node->fetch_element('fs')->fetch_with_id($label) ;
-
-            if (@comments) {
-                $logger->debug("Annotation: @comments\n");
-                $fs_obj->annotation(@comments);
-            }
-
-            my $load_line = "fs_vfstype=$type fs_spec=$device fs_file=$mount_point "
-                          . "fs_freq=$dump fs_passno=$pass" ;
-            $logger->debug("Loading:$load_line\n");
-            $fs_obj->load(step => $load_line, check => $check) ;
-
-            # now load fs options
-            $logger->trace("fs_type $type options is $options");
-            my @options = split /,/,$options ;
-            map {
-                $_ = $opt_r_translate{$_} if defined $opt_r_translate{$_};
-                s/no(.*)/$1=0/ ;
-                $_ .= '=1' unless /=/ ;
-            } @options ;
-            
-            $logger->debug("Loading:@options");
-            $fs_obj->fetch_element('fs_mntopts')->load (step => "@options", check => $check) ;
-
-            @comments = () ;
+        if ($comment) {
+            $logger->debug("Annotation: $comment\n");
+            $fs_obj->annotation($comment);
         }
-    }
 
-    return 1 ;
+        my $load_line = "fs_vfstype=$type fs_spec=$device fs_file=$mount_point "
+          . "fs_freq=$dump fs_passno=$pass";
+        $logger->debug("Loading:$load_line\n");
+        $fs_obj->load( step => $load_line, check => $check );
+
+        # now load fs options
+        $logger->trace("fs_type $type options is $options");
+        my @options = split /,/, $options;
+        map {
+            $_ = $opt_r_translate{$_} if defined $opt_r_translate{$_};
+            s/no(.*)/$1=0/;
+            $_ .= '=1' unless /=/;
+        } @options;
+
+        $logger->debug("Loading:@options");
+        $fs_obj->fetch_element('fs_mntopts')
+          ->load( step => "@options", check => $check );
+    }
+    return 1;
 }
 
 sub write {
@@ -119,32 +109,17 @@ sub write {
 
     croak "Undefined file handle to write" unless defined $ioh;
 
-    $ioh->print("## This file was written by Config::Model\n");
-    $ioh->print("## You may modify the content of this file. Configuration \n");
-    $ioh->print("## modifications will be preserved. Modifications in\n");
-    $ioh->print("## comments may be mangled.\n##\n");
-
-    # write global comment
-    my $global_note = $node->annotation ;
-    if ($global_note) {
-        map { $ioh->print("# $_\n") } split /\n/,$global_note ;
-        $ioh->print("\n") ;
-    }
+    $self->write_global_comment($ioh,'#') ;
 
     # Using Config::Model::ObjTreeScanner would be overkill
     foreach my $line_obj ($node->fetch_element('fs')->fetch_all ) {
-        # write line annotation
-        my $note = $line_obj->annotation ;
-        if ($note) {
-            map { $ioh->print("\n# $_") } split /\n/,$note ;
-            $ioh->print("\n");
-        }
-
-        $ioh->printf("%-30s %-25s %-6s %-10s %d %d\n",
+        my $d = sprintf("%-30s %-25s %-6s %-10s %d %d\n",
                      map ($line_obj->fetch_element_value($_), qw/fs_spec fs_file fs_vfstype/),
                      $self->option_string($line_obj->fetch_element('fs_mntopts')) ,
                      map ($line_obj->fetch_element_value($_) , qw/fs_freq fs_passno/),
                     );
+        $self->write_data_and_comments($ioh,'#',$d, $line_obj->annotation) ;
+        
     }
 
     return 1;
@@ -170,7 +145,7 @@ sub option_string {
     return join',',@options ;
 }
 
-no Moose ;
+no Any::Moose ;
 __PACKAGE__->meta->make_immutable ;
 
 1;
@@ -183,31 +158,23 @@ Config::Model::Backend::Fstab - Read and write config from fstab file
 
 =head1 VERSION
 
-version 1.226
+version 1.250
 
 =head1 SYNOPSIS
 
-  # model declaration
-  name => 'Fstab',
-
-  read_config  => [
-                    { backend => 'fstab' , 
-                      config_dir => '/etc/',
-                      file  => 'fstab', 
-                    }
-                  ],
-
-   element => ...
-  ) ;
-
+No synopsis. This class is dedicated to configuration class C<Fstab>
 
 =head1 DESCRIPTION
 
 This module is used directly by L<Config::Model> to read or write the
-content of a configuration tree written with fstab syntax in
+content of a configuration tree written with C<fstab> syntax in
 C<Config::Model> configuration tree. Typically this backend will 
 be used to read and write C</etc/fstab>.
 
+=head1 Comments in file_path
+
+This backend is able to read and write comments in the C</etc/fstab> file.
+
 =head1 STOP
 
 The documentation below describes methods that are currently used only by 
@@ -233,7 +200,7 @@ When a file is read,  C<read()> will return 1.
 =head2 write ( io_handle => ... )
 
 Of all parameters passed to this write call-back, only C<io_handle> is
-used. This parameter must be L<IO::File> object alwritey opened for
+used. This parameter must be L<IO::File> object already opened for
 write. 
 
 C<write()> will return 1.
@@ -248,4 +215,4 @@ L<Config::Model>,
 L<Config::Model::AutoRead>, 
 L<Config::Model::Backend::Any>, 
 
-=cut
\ No newline at end of file
+=cut
