@@ -1,13 +1,13 @@
 #
 # This file is part of Config-Model
 #
-# This software is Copyright (c) 2010 by Dominique Dumont, Krzysztof Tyszecki.
+# This software is Copyright (c) 2011 by Dominique Dumont, Krzysztof Tyszecki.
 #
 # This is free software, licensed under:
 #
 #   The GNU Lesser General Public License, Version 2.1, February 1999
 #
-#    Copyright (c) 2005-2010 Dominique Dumont.
+#    Copyright (c) 2005-2011 Dominique Dumont.
 #
 #    This file is part of Config-Model.
 #
@@ -27,16 +27,18 @@
 
 package Config::Model::CheckList ;
 BEGIN {
-  $Config::Model::CheckList::VERSION = '1.226';
+  $Config::Model::CheckList::VERSION = '1.250';
 }
 use Config::Model::Exception ;
 use Config::Model::IdElementReference ;
+use Config::Model::Warper ;
 use warnings ;
 use Carp;
 use strict;
 use Log::Log4perl qw(get_logger :levels);
+use Storable qw/dclone/;
 
-use base qw/Config::Model::WarpedThing/ ;
+use base qw/Config::Model::AnyThing/ ;
 
 
 my $logger = get_logger("Tree::Element::CheckList") ;
@@ -47,29 +49,49 @@ Config::Model::CheckList - Handle check list element
 
 =head1 VERSION
 
-version 1.226
+version 1.250
 
 =head1 SYNOPSIS
 
- $model ->create_config_class 
-  (
-   ...
-   element 
-   => [ 
-       check_list 
-       => { type => 'check_list',
-            choice => [ 'A', 'B', 'C', 'D' ],
-            help   => { A => 'A effect is this',
-                        D => 'D does that',
-                      }
-          },
-       check_list_refering_to_another_hash 
-       => { type => 'check_list',
-            refer_to => '- foobar'
-          },
+ use Config::Model;
+ use Log::Log4perl qw(:easy);
+ Log::Log4perl->easy_init($WARN);
 
-      ]
-  ) ;
+ # define configuration tree object
+ my $model = Config::Model->new;
+ $model->create_config_class(
+    name => "MyClass",
+
+    element => [
+
+        # type check_list will use Config::Model::CheckList
+        my_check_list => {
+            type   => 'check_list',
+            choice => [ 'A', 'B', 'C', 'D' ],
+            help   => {
+                A => 'A effect is this',
+                D => 'D does that',
+            }
+        },
+    ],
+ );
+
+ my $inst = $model->instance( root_class_name => 'MyClass' );
+
+ my $root = $inst->config_root;
+
+ # put data
+ $root->load( step => 'my_check_list=A' ); 
+
+ my $obj = $root->grab('my_check_list');
+
+ my $v = $root->grab_value('my_check_list');
+ print "check_list value '$v' with help '", $obj->get_help($v), "'\n";
+
+ # more data
+ $obj->check('D');
+ $v = $root->grab_value('my_check_list');
+ print "check_list new value is '$v'\n";   # prints check_list new value is 'A,D'
 
 =head1 DESCRIPTION
 
@@ -87,7 +109,7 @@ A fixed list (with the C<choice> parameter)
 
 =item *
 
-A dynamic list where the available choise are the keys of another hash
+A dynamic list where the available choice are the keys of another hash
 of the configuration tree. See L</"Choice reference"> for details.
 
 =back
@@ -131,16 +153,18 @@ sub new {
 	$self->submit_to_refer_to() ;
     }
 
-    $self->{backup}  = \%args ;
+    $self->{backup}  = dclone (\%args) ;
 
     $self->set_properties() ; # set will use backup data
 
     if (defined $warp_info) {
-	$self->check_warp_args( \@allowed_warp_params, $warp_info) ;
+        $self->{warper} = Config::Model::Warper->new (
+            warped_object => $self,
+            %$warp_info ,
+            allowed => \@allowed_warp_params
+        ) ;
     }
 
-    $self->submit_to_warp($self->{warp}) if $self->{warp} ;
-
     $self->cl_init ;
 
     $logger->info("Created check_list element $self->{element_name}");
@@ -300,7 +324,8 @@ sub set_properties {
     # merge data passed to the constructor with data passed to set
     my %args = (%{$self->{backup}},@_ );
 
-    $self->set_owner_element_property ( \%args );
+    # these are handled by Node or Warper
+    map { delete $args{$_} } qw/level experience/ ;
 
     $self->{ordered} = delete $args{ordered} || 0 ;
 
@@ -428,7 +453,7 @@ and C<my_hash2> and C<my_hash3> configuration parameter:
 A check list where the available choices are the specified choice and
 the choice of C<refer_to_3_lists> and a hash whose name is specified
 by the value of the C<indirection> configuration parameter (this
-example is admitedly convoluted):
+example is admittedly convoluted):
 
 
        refer_to_check_list_and_choice
@@ -457,9 +482,18 @@ sub submit_to_refer_to {
 	  -> new ( computed_refer_to => $self->{computed_refer_to} ,
 		   config_elt => $self,
 		 ) ;
+        my $var = $self->{computed_refer_to}{variables} ;
+
 	# refer_to registration is done for all element that are used as
 	# variable for complex reference (ie '- $foo' , {foo => '- bar'} )
-	$self->register_in_other_value($self->{computed_refer_to}{variables}) ;
+        foreach my $path (values %$var) {
+            # is ref during test case
+            #print "path is '$path'\n";
+            next if $path =~ /\$/ ; # next if path also contain a variable
+            my $master = $self->grab($path);
+            next unless $master->can('register_dependency');
+            $master->register_dependency($self) ;
+        }
     }
     else {
 	croak "checklist submit_to_refer_to: undefined refer_to or computed_refer_to" ;
@@ -501,11 +535,11 @@ sub cargo_type {
 =head2 check ( ... )
 
 Set choice. Parameter is either a list of choices to set or 
-a list ref and some optional parmeter. I.e:
+a list ref and some optional parameter. I.e:
 
   check (\@list, check => 'skip') ;
 
-C<check> parameter decide on behavior in case of unvalid
+C<check> parameter decide on behavior in case of invalid
 choice value: either die (if yes) or discard bad value (if skip)
 
 =cut
@@ -564,11 +598,11 @@ sub store {
 =head2 uncheck (...)
 
 Unset choice. Parameter is either a list of choices to unset or 
-a list ref and some optional parmeter. I.e:
+a list ref and some optional parameter. I.e:
 
   uncheck (\@list, check => 'skip') ;
 
-C<check> parameter decide on behavior in case of unvalid
+C<check> parameter decide on behavior in case of invalid
 choice value: either die (if yes) or discard bad value (if skip)
 
 =cut
@@ -590,7 +624,7 @@ sub uncheck {
 
 Return 1 if the given C<choice> was set. Returns 0 otherwise.
 
-C<check> parameter decide on behavior in case of unvalid
+C<check> parameter decide on behavior in case of invalid
 choice value: either die (if yes) or discard bad value (if skip)
 
 C<mode> is either: custom standard preset default upstream_default
@@ -1053,6 +1087,9 @@ sub move_down {
     }
 }
 
+# dummy to match Value call
+sub warning_msg { '' } 
+
 1;
 
 __END__
@@ -1071,4 +1108,4 @@ L<Config::Model::ListId>,
 L<Config::Model::HashId>,
 L<Config::Model::Value>
 
-=cut
\ No newline at end of file
+=cut
