@@ -1,7 +1,7 @@
 #
 # This file is part of Config-Model
 #
-# This software is Copyright (c) 2010 by Dominique Dumont, Krzysztof Tyszecki.
+# This software is Copyright (c) 2011 by Dominique Dumont, Krzysztof Tyszecki.
 #
 # This is free software, licensed under:
 #
@@ -28,7 +28,7 @@
 
 package Config::Model::Describe;
 BEGIN {
-  $Config::Model::Describe::VERSION = '1.226';
+  $Config::Model::Describe::VERSION = '1.250';
 }
 use Carp;
 use strict;
@@ -44,28 +44,61 @@ Config::Model::Describe - Provide a description of a node element
 
 =head1 VERSION
 
-version 1.226
+version 1.250
 
 =head1 SYNOPSIS
 
- use Config::Model ;
+ use Config::Model;
+ use Log::Log4perl qw(:easy);
+ Log::Log4perl->easy_init($WARN);
 
- # create your config model
- my $model = Config::Model -> new ;
- $model->create_config_class( ... ) ;
+ # define configuration tree object
+ my $model = Config::Model->new;
+  $model->create_config_class(
+    name    => "Foo",
+    element => [
+        [qw/foo bar/] => {
+            type       => 'leaf',
+            value_type => 'string'
+        },
+    ]
+ ); 
+ $model ->create_config_class (
+    name => "MyClass",
 
- # create instance
- my $inst = $model->instance (root_class_name => 'FooBar', 
-			      instance_name => 'test1');
+    element => [ 
 
- # create root of config
- my $root = $inst -> config_root ;
+        [qw/foo bar/] => {
+            type       => 'leaf',
+            value_type => 'string'
+        },
+        hash_of_nodes => {
+            type       => 'hash',     # hash id
+            index_type => 'string',
+            cargo      => {
+                type              => 'node',
+                config_class_name => 'Foo'
+            },
+        },
+    ],
+ ) ;
+
+ my $inst = $model->instance(root_class_name => 'MyClass' );
+
+ my $root = $inst->config_root ;
+
+ # put data
+ my $step = 'foo=FOO hash_of_nodes:fr foo=bonjour -
+   hash_of_nodes:en foo=hello ';
+ $root->load( step => $step );
 
  print $root->describe ;
 
- # or
-
- print $root->describe(element => 'foo' ) ;
+ ### prints
+ # name         value        type         comment                            
+ # foo          FOO          string                                          
+ # bar          [undef]      string                                          
+ # hash_of_nodes <Foo>        node hash    keys: "en" "fr"                    
 
 =head1 DESCRIPTION
 
@@ -73,7 +106,7 @@ This module is used directly by L<Config::Model::Node> to describe
 a node element. This module returns a human readable string that 
 shows the content of a configuration node.
 
-For instance (as showns by C<fstab> example:
+For instance (as shown by C<fstab> example:
 
  name         value        type         comment
  fs_spec      [undef]      string       mandatory
@@ -261,4 +294,4 @@ Dominique Dumont, (ddumont at cpan dot org)
 
 =head1 SEE ALSO
 
-L<Config::Model>,L<Config::Model::Node>,L<Config::Model::ObjTreeScanner>
\ No newline at end of file
+L<Config::Model>,L<Config::Model::Node>,L<Config::Model::ObjTreeScanner>
