@@ -1,12 +1,12 @@
-# 
+#
 # This file is part of Config-Model
-# 
-# This software is Copyright (c) 2010 by Dominique Dumont.
-# 
+#
+# This software is Copyright (c) 2011 by Dominique Dumont, Krzysztof Tyszecki.
+#
 # This is free software, licensed under:
-# 
+#
 #   The GNU Lesser General Public License, Version 2.1, February 1999
-# 
+#
 #    Copyright (c) 2006-2008,2010 Dominique Dumont.
 #
 #    This file is part of Config-Model.
@@ -26,6 +26,9 @@
 #    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
 
 package Config::Model::Loader;
+BEGIN {
+  $Config::Model::Loader::VERSION = '1.250';
+}
 use Carp;
 use strict;
 use warnings ;
@@ -33,8 +36,6 @@ use warnings ;
 use Config::Model::Exception ;
 use Log::Log4perl qw(get_logger :levels);
 
-our $VERSION="1.203";
-
 my $logger = get_logger("Loader") ;
 
 =head1 NAME
@@ -43,26 +44,85 @@ Config::Model::Loader - Load serialized data into config tree
 
 =head1 VERSION
 
-version 1.205
+version 1.250
 
 =head1 SYNOPSIS
 
- use Config::Model ;
+ use Config::Model;
+ use Log::Log4perl qw(:easy);
+ Log::Log4perl->easy_init($WARN);
 
- # create your config model
- my $model = Config::Model -> new ;
- $model->create_config_class( ... ) ;
+ # define configuration tree object
+ my $model = Config::Model->new;
+  $model->create_config_class(
+    name    => "Foo",
+    element => [
+        [qw/foo bar/] => {
+            type       => 'leaf',
+            value_type => 'string'
+        },
+    ]
+ ); 
+ 
+ $model ->create_config_class (
+    name => "MyClass",
 
- # create instance
- my $inst = $model->instance (root_class_name => 'FooBar', 
-			      instance_name => 'test1');
+    element => [ 
 
- # create root of config
- my $root = $inst -> config_root ;
+        [qw/foo bar/] => {
+            type       => 'leaf',
+            value_type => 'string'
+        },
+        hash_of_nodes => {
+            type       => 'hash',     # hash id
+            index_type => 'string',
+            cargo      => {
+                type              => 'node',
+                config_class_name => 'Foo'
+            },
+        },
+        [qw/lista listb/] => { 
+			      type => 'list',
+			      cargo =>  {type => 'leaf',
+					 value_type => 'string'
+					}
+			      },
+    ],
+ ) ;
 
- # put some data in config tree
- my $step = 'std_id:ab X=Bv - std_id:bc X=Av - a_string="toto tata"';
- $root->load( step => $step ) ;
+ my $inst = $model->instance(root_class_name => 'MyClass' );
+
+ my $root = $inst->config_root ;
+
+ # put data
+ my $step = 'foo=FOO hash_of_nodes:fr foo=bonjour -
+   hash_of_nodes:en foo=hello 
+   ! lista=foo,bar lista:2=baz
+     listb:0=foo listb:1=baz';
+ $root->load( step => $step );
+
+ print $root->describe,"\n" ;
+ # name         value        type         comment                            
+ # foo          FOO          string                                          
+ # bar          [undef]      string                                          
+ # hash_of_nodes <Foo>        node hash    keys: "en" "fr"                    
+ # lista        foo,bar,baz  list                                            
+ # listb        foo,baz      list                                            
+
+
+ # delete some data
+ $root->load( step => 'lista~2' );
+
+ print $root->describe(element => 'lista'),"\n" ;
+ # name         value        type         comment                            
+ # lista        foo,bar      list                                            
+
+ # append some data
+ $root->load( step => q!hash_of_nodes:en foo.=" world"! );
+
+ print $root->grab('hash_of_nodes:en')->describe(element => 'foo'),"\n" ;
+ # name         value        type         comment                            
+ # foo          "hello world" string                                          
 
 =head1 DESCRIPTION
 
@@ -90,7 +150,8 @@ sub new {
 
 =head1 load string syntax
 
-The string is made of the following items separated by spaces:
+The string is made of the following items (also called C<actions>)
+separated by spaces:
 
 =over 8
 
@@ -116,7 +177,7 @@ element with C<node> cargo_type)
 Go down using C<xxx> element and loop over the ids that match the regex.
 (For C<hash>)
 
-For instance, with OpenSsh model, you could do 
+For instance, with C<OpenSsh> model, you could do
 
  Host=~/.*.debian.org/ user='foo-guest'
 
@@ -138,7 +199,9 @@ For instance C<foo="a quoted string">. Note that you cannot embed
 double quote in this string. I.e C<foo="a \"quoted\" string"> will
 fail.
 
-C<foo=''> will set foo to C<undef>.
+=item xxx~
+
+Undef element C<xxx>
 
 =item xxx=z1,z2,z3
 
@@ -160,10 +223,20 @@ Will append C<zzz> value to current values (valid for C<leaf> elements).
 
 =item xxx#zzz or xxx:yyy#zzz
 
-Element annotation. Can be quoted or not quoted.
+Element annotation. Can be quoted or not quoted. Note that annotations are
+always placed at the end of an action item.
+
+I.e. C<foo#comment>, C<foo:bar#comment> or C<foo:bar=baz#comment> are valid.
+C<foo#comment:bar> is B<not> valid.
 
 =back
 
+=head2 Quotes
+
+You can surround indexes and values with double quotes. E.g.:
+
+  a_string="\"titi\" and \"toto\""
+
 =head1 Methods
 
 =head2 load ( ... )
@@ -208,27 +281,29 @@ sub load {
     my $step = delete $args{step} ;
     croak "load error: missing 'step' parameter" unless defined $step ;
 
-    my $experience = delete $args{experience} || 'master' ;
+    if (defined $args{permission}) {
+	carp "load: permission parameter is deprecated. Use experience";
+    }
+
+    my $experience = delete $args{experience} || delete $args{permission} || 'master' ;
     my $inst = $node->instance ;
 
     # tune value checking
-    my $tune_check 
-      = defined $args{check_store} 
-	and $args{check_store} == 0 ? 1 : 0 ;
-    $inst->push_no_value_check('store') if $tune_check ;
+    my $check = delete $args{check} || 'yes';
+    croak __PACKAGE__,"load: unexpected check $check" unless $check =~ /yes|no|skip/;
 
     # accept commands
     my $huge_string = ref $step ? join( ' ', @$step) : $step ;
 
     # do a split on ' ' but take quoted string into account
-    my @command = 
-      ( 
-       $huge_string =~ 
+    my @command =
+      (
+       $huge_string =~
        m/
          (         # begin of *one* command
           (?:        # group parts of a command (e.g ...:...=... )
            [^\s"]+  # match anything but a space and a quote
-           (?:        # begin quoted group 
+           (?:        # begin quoted group
              "         # begin of a string
               (?:        # begin group
                 \\"       # match an escaped quote
@@ -245,7 +320,7 @@ sub load {
 
     #print "command is ",join('+',@command),"\n" ;
 
-    my $ret = $self->_load($node, $experience, \@command,1) ;
+    my $ret = $self->_load($node, $check, $experience, \@command,1) ;
 
     if (@command) {
         my $str = "Error: command '@command' was not executed, you may have".
@@ -254,11 +329,14 @@ sub load {
 	    -> throw (
 		      error => $str,
 		      object => $node
-		     ) if $node->instance->get_value_check('store') ;
+		     ) if $check eq 'yes' ;
     }
 
-    # restore default value checks
-    $inst->pop_no_value_check  if $tune_check ;
+    if (%args) {
+	Config::Model::Exception::Internal->throw (
+	    error => __PACKAGE__." load: unexpected parameters: ".join(', ',keys %args) 
+	);
+    }
 
     return $ret ;
 }
@@ -271,11 +349,11 @@ sub _split_cmd {
 
 
     # do a split on ' ' but take quoted string into account
-    my @command = 
-      ( 
-       $cmd =~ 
+    my @command =
+      (
+       $cmd =~
        m!^
-	 (\w+)? # element name can be alone
+	 (\w[\w-]*)? # element name can be alone
 	 (?:
             (:|=~|~)       # action
             ( /[^/]+/      # regexp
@@ -283,11 +361,11 @@ sub _split_cmd {
 		$quoted_regexp
               |
 	      [^"#=\.~]+    # non action chars
-            )
+            )?
          )?
 	 (?:
             (=|.=)          # assign or append
-	    ( 
+	    (
               (?:
                 $quoted_regexp
                 | [^#\s]                # or non whitespace
@@ -296,7 +374,7 @@ sub _split_cmd {
 	 )?
 	 (?:
             \#              # optional annotation
-	    ( 
+	    (
               (?:
                  $quoted_regexp
                 | [^\s]                # or non whitespace
@@ -304,7 +382,7 @@ sub _split_cmd {
             )
 	 )?
          !gx
-       ) ; 
+       ) ;
     unquote (@command) ;
 
     return wantarray ? @command : \@command ;
@@ -321,7 +399,7 @@ my %load_dispatch = (
 
 # return 'done', 'root', 'up', 'error'
 sub _load {
-    my ($self, $node, $experience, $cmdref,$is_root) = @_ ;
+    my ($self, $node, $check, $experience, $cmdref,$is_root) = @_ ;
     $is_root ||= 0;
     my $node_name = "'".$node->name."'" ;
     $logger->debug("_load: called on node $node_name");
@@ -374,7 +452,7 @@ sub _load {
 			 );
 	}
 
-        unless (   $node->isa("Config::Model::Node") 
+        unless (   $node->isa("Config::Model::Node")
 		or $node->isa("Config::Model::WarpedNode")) {
 	    Config::Model::Exception::Load
 		-> throw (
@@ -396,7 +474,7 @@ sub _load {
 		-> throw (
 			  object => $node,
 			  element => $element_name,
-			 ) if $inst->get_value_check('store');
+			 ) if $check eq 'yes';
             unshift @$cmdref,$cmd ;
             return 'error' ;
 	}
@@ -407,19 +485,19 @@ sub _load {
 		-> throw (
 			  object => $node,
 			  element => $element_name
-			 ) if $inst->get_value_check('fetch_or_store') ;
+			 ) if $check eq 'yes';
             unshift @$cmdref,$cmd ;
             return 'error';
 	}
 
-        unless ($node->is_element_available(name => $element_name, 
+        unless ($node->is_element_available(name => $element_name,
 					    experience => $experience)) {
             Config::Model::Exception::RestrictedElement
 		-> throw (
 			  object => $node,
 			  element => $element_name,
 			  level => $experience,
-			 ) if $inst->get_value_check('fetch_or_store');
+			 ) if $check eq 'yes';
             unshift @$cmdref,$cmd ;
             return 'error' ;
 	}
@@ -432,39 +510,42 @@ sub _load {
 	  unless defined $method ;
 
 	$logger->debug("_load: calling $element_type loader on element $element_name") ;
-	my $target_obj ;
-	my $ret = $self->$method($node,$experience,
-				 \@instructions,$cmdref,\$target_obj) ;
-
-	# apply note on target object
-	if (defined $note) {
-	    if (defined $target_obj) {
-		$target_obj->annotation($note) ;
-	    }
-	    else {
-		Config::Model::Exception::Load
-		    -> throw (
-			      command => $cmd ,
-			      error => "Error: cannot set annotation with '".
-			      join("','", grep {defined $_ } @instructions)."'"
-			     );
-	    }
-	}
+	my $ret = $self->$method($node, $check,$experience, \@instructions,$cmdref) ;
 
 	if ($ret eq 'error' or $ret eq 'done') { return $ret; }
 	return 'root' if $ret eq 'root' and not $is_root ;
-	# ret eq up or ok -> go on with the loop 
+	# ret eq up or ok -> go on with the loop
     }
 
     return 'done' ;
 }
 
+sub _load_note {
+    my ( $self, $target_obj, $note, $instructions, $cmdref) = @_;
+
+    # apply note on target object
+    if ( defined $note ) {
+        if ( defined $target_obj ) {
+            $target_obj->annotation($note);
+        }
+        else {
+            Config::Model::Exception::Load->throw(
+                command => $$cmdref,
+                error   => "Error: cannot set annotation with '"
+                  . join( "','", grep { defined $_ } @$instructions ) . "'"
+            );
+        }
+    }
+}
+
 
 sub _walk_node {
-    my ($self,$node,$experience,$inst,$cmdref,$target_ref) = @_ ;
+    my ($self,$node, $check,$experience,$inst,$cmdref) = @_ ;
 
     my $element_name = shift @$inst ;
-    my $element = $$target_ref = $node -> fetch_element($element_name) ;
+    my $note = pop @$inst ;
+    my $element = $node -> fetch_element($element_name) ;
+    $self->_load_note($element, $note, $inst, $cmdref);
 
     my @left = grep {defined $_} @$inst ;
     if (@left) {
@@ -472,102 +553,120 @@ sub _walk_node {
 	    -> throw (
 		      command => $inst,
 		      error => "Don't know what to do with '@left' ".
-		      "for node element" . $element -> element_name
+		      "for node element " . $element -> element_name
 		     ) ;
     }
 
     $logger->info("Opening node element ", $element->name);
 
-    return $self->_load($element, $experience, $cmdref);
+    return $self->_load($element, $check, $experience, $cmdref);
 }
 
 sub unquote {
-    map { s/^"// && s/"$// && s/\\"/"/g if defined $_;  } @_ ;
+    map { s/^"// && s/"$// && s!\\"!"!g if defined $_;  } @_ ;
 }
 
 # used for list and check lists
 sub _load_list {
-    my ($self,$node,$experience,$inst,$cmdref,$target_ref) = @_ ;
-    my ($element_name,$action,$id,$subaction,$value) = @$inst ;
+    my ($self,$node, $check,$experience,$inst,$cmdref) = @_ ;
+    my ($element_name,$action,$id,$subaction,$value,$note) = @$inst ;
 
     my $element = $node -> fetch_element($element_name) ;
 
     my $elt_type   = $node -> element_type( $element_name ) ;
     my $cargo_type = $element->cargo_type ;
 
-    unless (defined $action or defined $subaction) {
-	$$target_ref = $element ;
+    if (defined $note and not defined $action and not defined $subaction) {
+        $self->_load_note($element, $note, $inst, $cmdref);
 	return 'ok';
     }
 
-    if (not defined $action and $subaction eq '=' 
+    if (not defined $action and defined $subaction and $subaction eq '='
 	and $cargo_type eq 'leaf'
        ) {
 	$logger->debug("_load_list: set whole list");
 	# valid for check_list or list
 	$logger->info("Setting $elt_type element ",$element->name,
 		      " with '$value'");
-	$element->load( $value ) ;
-	$$target_ref = $element ;
+	$element->load( $value , check => $check) ;
+        $self->_load_note($element, $note, $inst, $cmdref);
 	return 'ok';
     }
 
-    if ($elt_type eq 'list' and $action eq '~') {
+    if (not defined $action and defined $subaction ) {
+	Config::Model::Exception::Load
+	-> throw (
+		  object => $element,
+		  command => join('',grep (defined $_,@$inst)) ,
+		  error => "Wrong assignment with '$subaction' on "
+		  ."element type: $elt_type, cargo_type: $cargo_type"
+		 ) ;
+    }
+
+    if ($elt_type eq 'list' and defined $action and $action eq '~') {
 	# remove possible leading or trailing quote
 	$logger->debug("_load_list: removing id $id");
-	unquote ($id) ;
 	$element->remove($id) ;
 	return 'ok' ;
     }
 
-    if ($elt_type eq 'list' and $action eq ':') {
-	my $obj = $$target_ref = $element->fetch_with_id($id) ;
+    if ($elt_type eq 'list' and defined $action and $action eq ':') {
+	my $obj = $element->fetch_with_id(index => $id, check => $check) ;
+        $self->_load_note($obj, $note, $inst, $cmdref);
 
 	if ($cargo_type =~ /node/) {
 	    # remove possible leading or trailing quote
 	    $logger->debug("_load_list: calling _load on node id $id");
-	    unquote ($id) ;
-	    return $self->_load($obj, $experience, $cmdref);
+	    return $self->_load($obj, $check, $experience, $cmdref);
 	}
 
 	return 'ok' unless defined $subaction ;
 
 	if ($cargo_type =~ /leaf/) {
 	    $logger->debug("_load_list: calling _load_value on $cargo_type id $id");
-	    unquote($value) ;
-	    $self->_load_value($obj,$subaction,$value) 
+	    $self->_load_value($obj,$check,$subaction,$value)
 	      and return 'ok';
 	}
     }
 
+    my $a_str = defined $action ? $action : '<undef>' ;
 
     Config::Model::Exception::Load
 	-> throw (
 		  object => $element,
-		  command => join('',@$inst) ,
-		  error => "Wrong assignment with '$action' on "
+		  command =>  join ( '', map { $_ || '' } @$inst ),
+		  error => "Wrong assignment with '$a_str' on "
 		  ."element type: $elt_type, cargo_type: $cargo_type"
 		 ) ;
 
 }
 
 sub _load_hash {
-    my ($self,$node,$experience,$inst,$cmdref,$target_ref) = @_ ;
+    my ($self,$node,$check,$experience,$inst,$cmdref) = @_ ;
     my ($element_name,$action,$id,$subaction,$value,$note) = @$inst ;
 
     my $element = $node -> fetch_element($element_name) ;
     my $cargo_type = $element->cargo_type ;
 
-    unless (defined $action) {
-	$$target_ref = $element ;
+    if (defined $note and not defined $action) {
+        $self->_load_note($element, $note, $inst, $cmdref);
 	return 'ok';
     }
-
+    
+    if ( not defined $action ) {
+        Config::Model::Exception::Load->throw(
+            object  => $element,
+            command => join( '', map { $_ || '' } @$inst ),
+            error   => "Missing assignment on "
+              . "element type: hash, cargo_type: $cargo_type"
+        );
+    }
+    
     if ($action eq '=~') {
 	my @keys = $element->get_all_indexes;
 	my $ret ;
-	$logger->debug("_load_hash: looping with regex $id");
-	$id =~ s!^/!!; 
+	$logger->debug("_load_hash: looping with regex $id on keys @keys");
+	$id =~ s!^/!!;
 	$id =~ s!/$!! ;
 	my @saved_cmd = @$cmdref ;
 	foreach my $loop_id (grep /$id/,@keys) {
@@ -576,10 +675,10 @@ sub _load_hash {
 	    my $sub_elt =  $element->fetch_with_id($loop_id) ;
 	    if ($cargo_type =~ /node/) {
 		# remove possible leading or trailing quote
-		$ret = $self->_load($sub_elt, $experience, $cmdref);
+		$ret = $self->_load($sub_elt, $check,$experience, $cmdref);
 	    }
 	    elsif ($cargo_type =~ /leaf/) {
-		$ret = $self->_load_value($sub_elt,$subaction,$value) ;
+		$ret = $self->_load_value($sub_elt,$check,$subaction,$value) ;
 	    }
 	    else {
 		Config::Model::Exception::Load
@@ -591,7 +690,7 @@ sub _load_hash {
 			     ) ;
 	    }
 
-	    if ($ret eq 'error' or $ret eq 'done') { return $ret; }
+	    if ($ret eq 'error' or $ret eq 'root') { return $ret; }
 	}
 	return $ret ;
     }
@@ -600,23 +699,29 @@ sub _load_hash {
     if ($action eq '~') {
 	# remove possible leading or trailing quote
 	$logger->debug("_load_hash: deleting $id");
-	unquote ($id) ;
 	$element->delete($id) ;
 	return 'ok' ;
     }
 
-    my $obj = $$target_ref = $element->fetch_with_id($id) ;
+    my $obj = $element->fetch_with_id( index => $id , check => $check) ;
+    $self->_load_note($obj, $note, $inst, $cmdref);
 
     if ($action eq ':' and $cargo_type =~ /node/) {
 	# remove possible leading or trailing quote
 	$logger->debug("_load_hash: calling _load on node $id");
-	unquote ($id) ;
-	return $self->_load($obj, $experience, $cmdref);
+	if (defined $subaction) {
+            Config::Model::Exception::Load -> throw (
+		object => $element,
+                command => join('',@$inst) ,
+		error => qq!Hash assignment with '$action"$id"$subaction"$value"' on unexpected !
+		      ."cargo_type: $cargo_type"
+            ) ;
+	}
+	return $self->_load($obj,$check, $experience, $cmdref);
     }
     elsif ($action eq ':' and defined $subaction and $cargo_type =~ /leaf/) {
 	$logger->debug("_load_hash: calling _load_value on leaf $id");
-	unquote($id,$value) ;
-	$self->_load_value($obj,$subaction,$value)
+	$self->_load_value($obj,$check,$subaction,$value)
 	  and return 'ok';
     }
     elsif ($action eq ':' and defined $note) {
@@ -635,21 +740,26 @@ sub _load_hash {
 }
 
 sub _load_leaf {
-    my ($self,$node,$experience,$inst,$cmdref,$target_ref) = @_ ;
-    my ($element_name,$action,$id,$subaction,$value) = @$inst ;
+    my ($self,$node,$check,$experience,$inst,$cmdref) = @_ ;
+    my ($element_name,$action,$id,$subaction,$value,$note) = @$inst ;
 
-    my $element = $$target_ref = $node -> fetch_element($element_name) ;
-    unquote($value) ;
+    my $element = $node -> fetch_element($element_name) ;
+    $self->_load_note($element, $note, $inst, $cmdref);
+
+    if (defined $action and $action eq '~' and $element->isa('Config::Model::Value')) {
+        $logger->debug("_load_leaf: action '$action' deleting value");
+	$element->store(undef) ;
+    }
 
     return 'ok' unless defined $subaction ;
 
     if ($logger->is_debug) {
-        my $msg = $value ;
+        my $msg = defined $value ? $value : '<undef>';
         $msg =~ s/\n/\\n/g;
         $logger->debug("_load_leaf: action '$subaction' value '$msg'");
     }
 
-    return $self->_load_value($element,$subaction,$value)
+    return $self->_load_value($element,$check,$subaction,$value)
       or Config::Model::Exception::Load
 	-> throw (
 		  object => $element,
@@ -661,15 +771,15 @@ sub _load_leaf {
 }
 
 sub _load_value {
-    my ($self,$element,$subaction,$value) = @_ ;
+    my ($self,$element,$check,$subaction,$value) = @_ ;
 
-    $logger->debug("_load_value: action '$subaction' value '$value'");
+    $logger->debug("_load_value: action '$subaction' value '$value' check $check");
     if ($subaction eq '=' and $element->isa('Config::Model::Value')) {
-	$element->store($value) ;
+	$element->store(value => $value, check => $check) ;
     }
     elsif ($subaction eq '.=' and $element->isa('Config::Model::Value')) {
-	my $orig = $element->fetch() ;
-	$element->store($orig.$value) ;
+	my $orig = $element->fetch(check => $check) ;
+	$element->store(value => $orig.$value, check => $check) ;
     }
     else {
 	return undef ;
@@ -689,4 +799,4 @@ Dominique Dumont, (ddumont at cpan dot org)
 
 L<Config::Model>,L<Config::Model::Node>,L<Config::Model::Dumper>
 
-=cut
\ No newline at end of file
+=cut
